@@ -5,6 +5,16 @@ require 'zlib'
 require 'stringio'
 require 'base64'
 
+class VerbServlet < WEBrick::HTTPServlet::AbstractServlet
+  %w(HEAD GET POST PUT DELETE).each do |verb|
+    eval(<<-eomethod)
+      def do_#{verb}(req, res)
+        res.body = "method: #{verb}"
+      end
+    eomethod
+  end
+end
+
 class BasicAuthServlet < WEBrick::HTTPServlet::AbstractServlet
   def do_GET(req,res)
     htpd = WEBrick::HTTPAuth::Htpasswd.new('dot.htpasswd')
@@ -26,9 +36,40 @@ class BasicAuthServlet < WEBrick::HTTPServlet::AbstractServlet
   alias :do_POST :do_GET
 end
 
+class DigestAuthServlet < WEBrick::HTTPServlet::AbstractServlet
+  htpd = WEBrick::HTTPAuth::Htdigest.new('digest.htpasswd')
+  htpd.set_passwd('Blah', 'user', 'pass')
+  @@authenticator = WEBrick::HTTPAuth::DigestAuth.new({
+                                                   :UserDB => htpd,
+                                                   :Realm  => 'Blah',
+                                                   :Algorithm => 'MD5',
+                                                   :Logger => Logger.new(nil)
+  }
+  )
+  def do_GET(req,res)
+    def req.request_time; Time.now; end
+    def req.request_uri; '/digest_auth'; end
+    def req.request_method; "GET"; end
+
+    begin
+      @@authenticator.authenticate(req,res)
+      res.body = 'You are authenticated'
+    rescue WEBrick::HTTPStatus::Unauthorized => ex
+      res.status = 401
+    end
+    FileUtils.rm('digest.htpasswd') if File.exists?('digest.htpasswd')
+  end
+  alias :do_POST :do_GET
+end
+
 class HeaderServlet < WEBrick::HTTPServlet::AbstractServlet
   def do_GET(req, res)
     res['Content-Type'] = "text/html"
+
+    req.query.each do |x,y|
+      res[x] = y
+    end
+
     body = ''
     req.each_header do |k,v|
       body << "#{k}|#{v}\n"
@@ -114,6 +155,38 @@ class FileUploadTest < WEBrick::HTTPServlet::AbstractServlet
   end
 end
 
+class InfiniteRefreshTest < WEBrick::HTTPServlet::AbstractServlet
+  def do_GET(req, res)
+    res['Content-Type'] = req.query['ct'] || "text/html"
+    res.status = req.query['code'] ? req.query['code'].to_i : '302'
+    number = req.query['q'] ? req.query['q'].to_i : 0
+    res['Refresh'] = " 0;url=http://localhost/infinite_refresh?q=#{number + 1}\r\n";
+  end
+end
+
+class InfiniteRedirectTest < WEBrick::HTTPServlet::AbstractServlet
+  def do_GET(req, res)
+    res['Content-Type'] = req.query['ct'] || "text/html"
+    res.status = req.query['code'] ? req.query['code'].to_i : '302'
+    number = req.query['q'] ? req.query['q'].to_i : 0
+    res['Location'] = "/infinite_redirect?q=#{number + 1}"
+  end
+  alias :do_POST :do_GET
+end
+
+class RedirectTest < WEBrick::HTTPServlet::AbstractServlet
+  def do_GET(req, res)
+    res['Content-Type'] = req.query['ct'] || "text/html"
+    res.status = req.query['code'] ? req.query['code'].to_i : '302'
+    res['Location'] = "/verb"
+  end
+
+  alias :do_POST :do_GET
+  alias :do_HEAD :do_GET
+  alias :do_PUT :do_GET
+  alias :do_DELETE :do_GET
+end
+
 class ResponseCodeTest < WEBrick::HTTPServlet::AbstractServlet
   def do_GET(req, res)
     res['Content-Type'] = req.query['ct'] || "text/html"
@@ -128,6 +201,16 @@ class ResponseCodeTest < WEBrick::HTTPServlet::AbstractServlet
     end
   end
 end
+
+class HttpRefreshTest < WEBrick::HTTPServlet::AbstractServlet
+  def do_GET(req, res)
+    res['Content-Type'] = req.query['ct'] || "text/html"
+    refresh_time = req.query['refresh_time'] || 0
+    refresh_url = req.query['refresh_url'] || '/index.html'
+    res['Refresh'] = " #{refresh_time};url=#{refresh_url}\r\n";
+  end
+end
+
 class FormTest < WEBrick::HTTPServlet::AbstractServlet
   def do_GET(req, res)
     res.body = "<HTML><body>"
